// Package cmd implements the command line interface.
package cmd

import (
	"context"
	"fmt"
	"os"
	"os/signal"
	"syscall"

	"git.lyda.ie/kevin/units/web"
	"github.com/spf13/cobra"
	"github.com/spf13/viper"
)

var cfgFile string

// rootCmd represents the base command when called without any subcommands
var rootCmd = &cobra.Command{
	Use:   "units",
	Short: "Converts unit(s) to unit(s)",
	Long: `Converts any unit(s) to any other unit(s)
For example:

TODO: example 1
TODO: example 2
TODO: example 3`,
	Run: run,
}

func run(_ *cobra.Command, _ []string) {
	fmt.Printf("Server listening on '%s'\n", viper.GetString("listen"))
	server, err := web.New(viper.GetString("listen"))
	cobra.CheckErr(err)

	ctx, stop := signal.NotifyContext(context.Background(), os.Interrupt, syscall.SIGTERM)
	defer stop()

	server.Serve(ctx)
}

// Execute adds all child commands to the root command and sets flags appropriately.
// This is called by main.main(). It only needs to happen once to the rootCmd.
func Execute() {
	err := rootCmd.Execute()
	if err != nil {
		os.Exit(1)
	}
}

func init() {
	cobra.OnInitialize(initConfig)
	rootCmd.PersistentFlags().StringVar(&cfgFile, "config", "", "config file (default is $HOME/.units.yaml)")
	rootCmd.Flags().StringP("listen", "l", "0.0.0.0:8989", "host:port to listen on")
}

// initConfig reads in config file and ENV variables if set.
func initConfig() {
	if cfgFile != "" {
		// Use config file from the flag.
		viper.SetConfigFile(cfgFile)
	} else {
		// Find home directory.
		home, err := os.UserHomeDir()
		cobra.CheckErr(err)

		// Search config in home directory with name ".units" (without extension).
		viper.AddConfigPath(home)
		viper.SetConfigType("yaml")
		viper.SetConfigName(".units")
	}

	viper.AutomaticEnv() // read in environment variables that match
	viper.BindPFlags(rootCmd.Flags())

	// If a config file is found, read it in.
	if err := viper.ReadInConfig(); err == nil {
		fmt.Fprintln(os.Stderr, "Using config file:", viper.ConfigFileUsed())
	}
}
